##############################################################################
#
# Copyright (c) 2011 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Mongo  Tests"""
import doctest
import persistent
import pprint
import transaction
from pymongo import dbref, objectid

from mongopersist import interfaces, testing, datamanager

class Foo(persistent.Persistent):
    def __init__(self, name=None):
        self.name = name

class Bar(persistent.Persistent):
    _p_mongo_sub_object = True

def doctest_create_conflict_error():
    r"""create_conflict_error(): General Test

    Simple helper function to create a conflict error.

     >>> foo = Foo()
     >>> foo._p_serial = '\x00\x00\x00\x00\x00\x00\x00\x01'

     >>> datamanager.create_conflict_error(foo, {'_py_serial': 3})
     ConflictError: database conflict error
                    (oid None, class Foo, start serial 1, current serial 3)
    """

def doctest_Root():
    r"""Root: General Test

    This class represents the root(s) of the object tree. All roots are stored
    in a specified collection. Since the rooted object needs to immediately
    provide a data manager (jar), the operations on the DB root are not art of
    the transaction mechanism.

      >>> root = datamanager.Root(dm, DBNAME, 'proot')

    Initially the root is empty:

      >>> root.keys()
      []

    Let's now add an item:

      >>> foo = Foo()
      >>> root['foo'] = foo
      >>> root.keys()
      [u'foo']
      >>> root['foo'] == foo
      True

    Root objects can be overridden:

      >>> foo2 = Foo()
      >>> root['foo'] = foo2
      >>> root.keys()
      [u'foo']
      >>> root['foo'] == foo
      False

    And of course we can delete an item:

      >>> del root['foo']
      >>> root.keys()
      []
    """

def doctest_MongoDataManager_object_dump_load_reset():
    r"""MongoDataManager: dump(), load(), reset()

    The Mongo Data Manager is a persistent data manager that manages object
    states in a Mongo database accross Python transactions.

    There are several arguments to create the data manager, but only the
    pymongo connection is required:

      >>> dm = datamanager.MongoDataManager(
      ...     conn,
      ...     detect_conflicts=True,
      ...     default_database = DBNAME,
      ...     root_database = DBNAME,
      ...     root_collection = 'proot',
      ...     name_map_collection = 'coll_pypath_map',
      ...     conflict_error_factory = datamanager.create_conflict_error)

    There are two convenience methods that let you serialize and de-serialize
    objects explicitly:

      >>> foo = Foo()
      >>> dm.dump(foo)
      DBRef('mongopersist.tests.test_datamanager.Foo',
            ObjectId('4eb2eb7437a08e0156000000'),
            'mongopersist_test')

    Let's now reset the data manager, so we do not hit a cache while loading
    the object again:

      >>> dm.reset()

    We can now load the object:

      >>> foo2 = dm.load(foo._p_oid)
      >>> foo == foo2
      False
      >>> foo._p_oid = foo2._p_oid
    """

def doctest_MongoDataManager_set_state():
    r"""MongoDataManager: set_state()

    This method loads and sets the state of an object and joins the
    transaction.

      >>> foo = Foo(u'foo')
      >>> ref = dm.dump(foo)

      >>> dm.reset()
      >>> dm._needs_to_join
      True

      >>> foo2 = Foo()
      >>> foo2._p_oid = ref
      >>> dm.setstate(foo2)
      >>> foo2.name
      u'foo'

      >>> dm._needs_to_join
      False
    """

def doctest_MongoDataManager_oldstate():
    r"""MongoDataManager: oldstate()

    Loads the state of an object for a given transaction. Since we are not
    supporting history, this always raises a key error as documented.

      >>> foo = Foo(u'foo')
      >>> dm.oldstate(foo, '0')
      Traceback (most recent call last):
      ...
      KeyError: '0'
    """

def doctest_MongoDataManager_register():
    r"""MongoDataManager: register()

    Registers an object to be stored.

      >>> dm._needs_to_join
      True
      >>> len(dm._registered_objects)
      0

      >>> foo = Foo(u'foo')
      >>> dm.register(foo)

      >>> dm._needs_to_join
      False
      >>> len(dm._registered_objects)
      1

   But there are no duplicates:

      >>> dm.register(foo)
      >>> len(dm._registered_objects)
      1
    """

def doctest_MongoDataManager_abort():
    r"""MongoDataManager: abort()

    Aborts a transaction, which clears all object and transaction registrations:

      >>> dm._registered_objects = [Foo()]
      >>> dm._needs_to_join = False

      >>> dm.abort(transaction.get())

      >>> dm._needs_to_join
      True
      >>> len(dm._registered_objects)
      0
    """

def doctest_MongoDataManager_commit():
    r"""MongoDataManager: commit()

    Contrary to what the name suggests, this is the commit called during the
    first phase of a two-phase commit. Thus, for all practically purposes,
    this method merely checks whether the commit would potentially fail.

    This means, if conflict detection is disabled, this method does nothing.

      >>> dm.detect_conflicts
      False
      >>> dm.commit(transaction.get())

    Let's now turn on conflict detection:

      >>> dm.detect_conflicts = True

    For new objects (not having an oid), it always passes:

      >>> dm.reset()
      >>> dm._registered_objects = [Foo()]
      >>> dm.commit(transaction.get())

    If the object has an oid, but is not found in the DB, we also just pass,
    because the object will be inserted.

      >>> foo = Foo()
      >>> foo._p_oid =  dbref.DBRef(
      ...     'mongopersist.tests.test_datamanager.Foo',
      ...     objectid.ObjectId('4eb2eb7437a08e0156000000'),
      ...     'mongopersist_test')

      >>> dm.reset()
      >>> dm._registered_objects = [foo]
      >>> dm.commit(transaction.get())

    Let's now store an object and make sure it does not conflict:

      >>> foo = Foo()
      >>> ref = dm.dump(foo)
      >>> ref
      DBRef('mongopersist.tests.test_datamanager.Foo',
            ObjectId('4eb3468037a08e1b74000000'),
            'mongopersist_test')

      >>> dm.reset()
      >>> dm._registered_objects = [foo]
      >>> dm.commit(transaction.get())

    Next, let's cause a conflict byt simulating a conflicting transaction:

      >>> dm.reset()
      >>> foo2 = dm.load(ref)
      >>> foo2.name = 'foo2'
      >>> transaction.commit()

      >>> dm.reset()
      >>> dm._registered_objects = [foo]
      >>> dm.commit(transaction.get())
      Traceback (most recent call last):
      ...
      ConflictError: database conflict error
          (oid DBRef('mongopersist.tests.test_datamanager.Foo',
                     ObjectId('4eb3499637a08e1c5a000000'),
                     'mongopersist_test'),
           class Foo, start serial 1, current serial 2)
    """

def doctest_MongoDataManager_tpc_begin():
    r"""MongoDataManager: tpc_begin()

    This is a non-op for the mongo data manager.

      >>> dm.tpc_begin(transaction.get())
    """

def doctest_MongoDataManager_tpc_vote():
    r"""MongoDataManager: tpc_vote()

    This is a non-op for the mongo data manager.

      >>> dm.tpc_vote(transaction.get())
    """

def doctest_MongoDataManager_tpc_finish():
    r"""MongoDataManager: tpc_finish()

    This method finishes the two-phase commit. So let's store a simple object:

      >>> foo = Foo()
      >>> dm.detect_conflicts = True
      >>> dm._registered_objects = [foo]
      >>> dm.tpc_finish(transaction.get())
      >>> foo._p_serial
      '\x00\x00\x00\x00\x00\x00\x00\x01'

    Note that objects cannot be stored twice in the same transation:

      >>> dm.reset()
      >>> dm._registered_objects = [foo, foo]
      >>> dm.tpc_finish(transaction.get())
      >>> foo._p_serial
      '\x00\x00\x00\x00\x00\x00\x00\x02'

    Also, when a persistent sub-object is stored that does not want its own
    document, then its parent is stored instead, still avoiding dual storage.

      >>> dm.reset()
      >>> foo2 = dm.load(foo._p_oid)
      >>> foo2.bar = Bar()

      >>> dm.tpc_finish(transaction.get())
      >>> foo2._p_serial
      '\x00\x00\x00\x00\x00\x00\x00\x03'

      >>> dm.reset()
      >>> foo3 = dm.load(foo._p_oid)
      >>> dm._registered_objects = [foo3.bar, foo3]
      >>> dm.tpc_finish(transaction.get())
      >>> foo3._p_serial
      '\x00\x00\x00\x00\x00\x00\x00\x04'

    """

def doctest_MongoDataManager_tpc_abort():
    r"""MongoDataManager: tpc_abort()

    Aborts a two-phase commit. This is simply the same as the regular abort.

      >>> dm._registered_objects = [Foo()]
      >>> dm._needs_to_join = False

      >>> dm.tpc_abort(transaction.get())

      >>> dm._needs_to_join
      True
      >>> len(dm._registered_objects)
      0
    """

def doctest_MongoDataManager_sortKey():
    r"""MongoDataManager: sortKey()

    The data manager's sort key is trivial.

      >>> dm.sortKey()
      ('MongoDataManager', 0)
    """

def doctest_processSpec():
    r"""processSpec(): General test

    A simple helper function that returns the spec itself if no
    IMongoSpecProcessor adapter is registered.

      >>> from zope.testing.cleanup import CleanUp as PlacelessSetup
      >>> PlacelessSetup().setUp()


      >>> datamanager.processSpec('a_collection', {'life': 42})
      {'life': 42}

    Now let's register an adapter

      >>> class Processor(object):
      ...     def __init__(self, context):
      ...         pass
      ...     def process(self, collection, spec):
      ...         print 'passed in:', collection, spec
      ...         return {'life': 24}

      >>> import zope.interface
      >>> from zope.component import provideAdapter
      >>> provideAdapter(Processor, (zope.interface.Interface,), interfaces.IMongoSpecProcessor)

    And see what happens on processSpec:

      >>> datamanager.processSpec('a_collection', {'life': 42})
      passed in: a_collection {'life': 42}
      {'life': 24}

    We get the processed spec in return.


      >>> PlacelessSetup().tearDown()

    """

def test_suite():
    return doctest.DocTestSuite(
        setUp=testing.setUp, tearDown=testing.tearDown,
        checker=testing.checker,
        optionflags=testing.OPTIONFLAGS)



##############################################################################
#
# Copyright (c) 2011 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Mongo Persistence Interfaces"""
from __future__ import absolute_import
import datetime
import persistent.interfaces
import transaction.interfaces
import types
import zope.interface
import zope.schema
from pymongo import objectid, dbref

MONGO_NATIVE_TYPES = (
    int, float, unicode, datetime.datetime, types.NoneType,
    objectid.ObjectId, dbref.DBRef)

class ConflictError(transaction.interfaces.TransientError):

    def __init__(self, message=None, object=None, serials=None):
        self.message = message or "database conflict error"
        self.object = object
        self.serials = serials

    @property
    def new_serial(self):
        return self.serials[0]

    @property
    def old_serial(self):
        return self.serials[1]

    def __str__(self):
        extras = [
            'oid %s' %self.object._p_oid,
            'class %s' %self.object.__class__.__name__,
            'start serial %s' %self.old_serial,
            'current serial %s' %self.new_serial]
        return "%s (%s)" % (self.message, ", ".join(extras))

    def __repr__(self):
        return '%s: %s' %(self.__class__.__name__, self)


class CircularReferenceError(Exception):
    pass

class IObjectSerializer(zope.interface.Interface):
    """An object serializer allows for custom serialization output for
    objects."""

    def can_read(state):
        """Returns a boolean indicating whether this serializer can deserialize
        this state."""

    def get_object(state):
        """Convert the state to an object."""

    def can_write(obj):
        """Returns a boolean indicating whether this serializer can serialize
        this object."""

    def get_state(obj):
        """Convert the object to a state/document."""


class IObjectWriter(zope.interface.Interface):
    """The object writer stores an object in the database."""

    def get_non_persistent_state(obj, seen):
        """Convert a non-persistent object to a Mongo state/document."""

    def get_persistent_state(obj, seen):
        """Convert a persistent object to a Mongo state/document."""

    def get_state(obj, seen=None):
        """Convert an arbitrary object to a Mongo state/document.

        A ``CircularReferenceError`` is raised, if a non-persistent loop is
        detected.
        """

    def store(obj):
        """Store an object in the database."""


class IObjectReader(zope.interface.Interface):
    """The object reader reads an object from the database."""

    def resolve(path):
        """Resolve a path to a class.

        The path can be any string. It is the responsibility of the resolver
        to maintain the mapping from path to class.
        """

    def get_object(state, obj):
        """Get an object from the given state.

        The ``obj`` is the Mongo document of which the created object is part
        of.
        """

    def set_ghost_state(obj):
        """Convert a ghosted object to an active object by loading its state.
        """

    def get_ghost(coll_name, oid):
        """Get the ghosted version of the object.
        """


class IMongoDataManager(persistent.interfaces.IPersistentDataManager):
    """A persistent data manager that stores data in Mongo."""

    root = zope.interface.Attribute(
        """Get the root object, which is a mapping.""")

    def reset():
        """Reset the datamanager for the next transaction."""

    def dump(obj):
        """Store the object to Mongo and return its DBRef."""

    def load(dbref):
        """Load the object from Mongo by using its DBRef.

        Note: The returned object is in the ghost state.
        """


class IMongoConnectionPool(zope.interface.Interface):
    """MongoDB connection pool"""

    connection = zope.interface.Attribute('MongoDBConnection instance')

    host = zope.schema.TextLine(
        title=u'MongoDB Server Hostname (without protocol)',
        description=u'MongoDB Server Hostname or IPv4 address',
        default=u'localhost',
        required=True)

    port = zope.schema.Int(
        title=u'MongoDB Server Port',
        description=u'MongoDB Server Port',
        default=27017,
        required=True)


class IMongoDataManagerProvider(zope.interface.Interface):
    """Utility to get a mongo data manager.

    Implementations of this utility ususally maintain connection information
    and ensure that there is one consistent datamanager per thread.
    """

    def get():
        """Return a mongo data manager."""


class IMongoSpecProcessor(zope.interface.Interface):
    """An adapter to process find/update spec's"""

    def process(collection, spec):
        """return the processed spec here"""


##############################################################################
#
# Copyright (c) 2011 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Mongo Persistence Doc Tests"""
import doctest

import ZODB
import ZODB.DemoStorage
import persistent
import pymongo
import re
import transaction
import zope.component
import zope.interface
import zope.lifecycleevent
from pprint import pprint
from zope.app.testing import placelesssetup
from zope.container import contained, btree
from zope.testing import module, renormalizing

from mongopersist import datamanager, interfaces, serialize
from mongopersist.zope import container

class ApplicationRoot(container.SimpleMongoContainer):
    _p_mongo_collection = 'root'

    def __repr__(self):
        return '<ApplicationRoot>'

class SimplePerson(contained.Contained, persistent.Persistent):
    _p_mongo_collection = 'person'

    def __init__(self, name):
        self.name = name

    def __str__(self):
        return self.name

    def __repr__(self):
        return '<%s %s>' %(self.__class__.__name__, self)

class Person(container.MongoContained, SimplePerson):
    pass


def doctest_SimpleMongoContainer_basic():
    """SimpleMongoContainer: basic

      >>> cn = 'mongopersist.zope.container.SimpleMongoContainer'

    Let's make sure events are fired correctly:

      >>> zope.component.provideHandler(handleObjectModifiedEvent)

    Let's add a container to the root:

      >>> dm.reset()
      >>> dm.root['c'] = container.SimpleMongoContainer()

      >>> db = dm._conn[DBNAME]
      >>> pprint(list(db[cn].find()))
      [{u'_id': ObjectId('4e7ea146e13823316f000000'), u'data': {}}]

    As you can see, the serialization is very clean. Next we add a person.

      >>> dm.root['c'][u'stephan'] = SimplePerson(u'Stephan')
      ContainerModifiedEvent: <...SimpleMongoContainer ...>
      >>> dm.root['c'].keys()
      [u'stephan']
      >>> dm.root['c'][u'stephan']
      <SimplePerson Stephan>

      >>> dm.root['c']['stephan'].__parent__
      <mongopersist.zope.container.SimpleMongoContainer object at 0x7fec50f86500>
      >>> dm.root['c']['stephan'].__name__
      u'stephan'

    You can also access objects using the ``get()`` method of course:

      >>> stephan = dm.root['c'].get(u'stephan')
      >>> stephan.__parent__
      <mongopersist.zope.container.SimpleMongoContainer object at 0x7fec50f86500>
      >>> stephan.__name__
      u'stephan'

    Let's commit and access the data again:

      >>> transaction.commit()

      >>> pprint(list(db['person'].find()))
      [{u'__name__': u'stephan',
        u'__parent__':
            DBRef(u'mongopersist.zope.container.SimpleMongoContainer',
                  ObjectId('4e7ddf12e138237403000000'),
                  u'mongopersist_container_test'),
        u'_id': ObjectId('4e7ddf12e138237403000000'),
        u'name': u'Stephan'}]

      >>> dm.root['c'].keys()
      [u'stephan']
      >>> dm.root['c']['stephan'].__parent__
      <mongopersist.zope.container.SimpleMongoContainer object at 0x7fec50f86500>
      >>> dm.root['c']['stephan'].__name__
      u'stephan'

      >>> dm.root['c'].items()
      [(u'stephan', <SimplePerson Stephan>)]

      >>> dm.root['c'].values()
      [<SimplePerson Stephan>]

    Now remove the item:

      >>> del dm.root['c']['stephan']
      ContainerModifiedEvent: <...SimpleMongoContainer ...>

    The changes are immediately visible.

      >>> dm.root['c'].keys()
      []
      >>> dm.root['c']['stephan']
      Traceback (most recent call last):
      ...
      KeyError: 'stephan'

    Make sure it is really gone after committing:

      >>> transaction.commit()
      >>> dm.root['c'].keys()
      []
    """


def doctest_MongoContainer_basic():
    """MongoContainer: basic

    Let's make sure events are fired correctly:

      >>> zope.component.provideHandler(handleObjectModifiedEvent)

    Let's add a container to the root:

      >>> transaction.commit()
      >>> dm.root['c'] = container.MongoContainer('person')

      >>> db = dm._conn[DBNAME]
      >>> pprint(list(db['mongopersist.zope.container.MongoContainer'].find()))
      [{u'_id': ObjectId('4e7ddf12e138237403000000'),
        u'_m_collection': u'person'}]

    It is unfortunate that the '_m_collection' attribute is set. This is
    avoidable using a sub-class.

      >>> dm.root['c'][u'stephan'] = Person(u'Stephan')
      ContainerModifiedEvent: <...MongoContainer ...>
      >>> dm.root['c'].keys()
      [u'stephan']
      >>> dm.root['c'][u'stephan']
      <Person Stephan>

      >>> dm.root['c']['stephan'].__parent__
      <mongopersist.zope.container.MongoContainer object at 0x7fec50f86500>
      >>> dm.root['c']['stephan'].__name__
      u'stephan'

    It is a feature of the container that the item is immediately available
    after assignment, but before the data is stored in the database. Let's
    commit and access the data again:

      >>> transaction.commit()

      >>> pprint(list(db['person'].find()))
      [{u'_id': ObjectId('4e7e9d3ae138232d7b000003'),
        u'key': u'stephan',
        u'name': u'Stephan',
        u'parent': DBRef(u'mongopersist.zope.container.MongoContainer',
                         ObjectId('4e7e9d3ae138232d7b000000'),
                         u'mongopersist_container_test')}]

      >>> dm.root['c'].keys()
      [u'stephan']
      >>> dm.root['c']['stephan'].__parent__
      <mongopersist.zope.container.MongoContainer object at 0x7fec50f86500>
      >>> dm.root['c']['stephan'].__name__
      'stephan'

    We get a usual key error, if an object does not exist:

      >>> dm.root['c']['roy']
      Traceback (most recent call last):
      ...
      KeyError: 'roy'

    Now remove the item:

      >>> del dm.root['c']['stephan']
      ContainerModifiedEvent: <...MongoContainer ...>

    The changes are immediately visible.

      >>> dm.root['c'].keys()
      []
      >>> dm.root['c']['stephan']
      Traceback (most recent call last):
      ...
      KeyError: 'stephan'

    Make sure it is really gone after committing:

      >>> transaction.commit()
      >>> dm.root['c'].keys()
      []
    """

def doctest_MongoContainer_constructor():
    """MongoContainer: constructor

    The constructor of the MongoContainer class has several advanced arguments
    that allow customizing the storage options.

      >>> transaction.commit()
      >>> c = container.MongoContainer(
      ...     'person',
      ...     database = 'testdb',
      ...     mapping_key = 'name',
      ...     parent_key = 'site')

    The database allows you to specify a custom database in which the items
    are located. Otherwise the datamanager's default database is used.

      >>> c._m_database
      'testdb'

    The mapping key is the key/attribute of the contained items in which their
    name/key within the mapping is stored.

      >>> c._m_mapping_key
      'name'

    The parent key is the key/attribute in which the parent reference is
    stored. This is used to suport multiple containers per Mongo collection.

      >>> c._m_parent_key
      'site'
    """
def doctest_MongoContainer_m_parent_key_value():
    r"""MongoContainer: _m_parent_key_value()

    This method is used to extract the parent refernce for the item.

      >>> c = container.MongoContainer('person')

    The default implementation requires the container to be in some sort of
    persistent store, though it does not care whether this store is Mongo or a
    classic ZODB. This feature allows one to mix and match ZODB and Mongo
    storage.

      >>> c._m_get_parent_key_value()
      Traceback (most recent call last):
      ...
      ValueError: _p_jar not found.

    Now the ZODB case:

      >>> c._p_jar = object()
      >>> c._p_oid = '\x00\x00\x00\x00\x00\x00\x00\x01'
      >>> c._m_get_parent_key_value()
      'zodb-0000000000000001'

    And finally the Mongo case:

      >>> c._p_jar = c._p_oid = None
      >>> dm.root['people'] = c
      >>> c._m_get_parent_key_value()
      <mongopersist.zope.container.MongoContainer object at 0x32deed8>

    In that final case, the container itself is returned, because upon
    serialization, we simply look up the dbref.
    """

def doctest_MongoContainer_many_items():
    """MongoContainer: many items

    Let's create an interesting set of data:

      >>> transaction.commit()
      >>> dm.root['people'] = container.MongoContainer('person')
      >>> dm.root['people'][u'stephan'] = Person(u'Stephan')
      >>> dm.root['people'][u'roy'] = Person(u'Roy')
      >>> dm.root['people'][u'roger'] = Person(u'Roger')
      >>> dm.root['people'][u'adam'] = Person(u'Adam')
      >>> dm.root['people'][u'albertas'] = Person(u'Albertas')
      >>> dm.root['people'][u'russ'] = Person(u'Russ')

    In order for find to work, the data has to be committed:

      >>> transaction.commit()

    Let's now search and receive documents as result:

      >>> sorted(dm.root['people'].keys())
      [u'adam', u'albertas', u'roger', u'roy', u'russ', u'stephan']
      >>> dm.root['people'][u'stephan']
      <Person Stephan>
      >>> dm.root['people'][u'adam']
      <Person Adam>
"""

def doctest_MongoContainer_find():
    """MongoContainer: find

    The Mongo Container supports direct Mongo queries. It does, however,
    insert the additional container filter arguments and can optionally
    convert the documents to objects.

    Let's create an interesting set of data:

      >>> transaction.commit()
      >>> dm.root['people'] = container.MongoContainer('person')
      >>> dm.root['people'][u'stephan'] = Person(u'Stephan')
      >>> dm.root['people'][u'roy'] = Person(u'Roy')
      >>> dm.root['people'][u'roger'] = Person(u'Roger')
      >>> dm.root['people'][u'adam'] = Person(u'Adam')
      >>> dm.root['people'][u'albertas'] = Person(u'Albertas')
      >>> dm.root['people'][u'russ'] = Person(u'Russ')

    In order for find to work, the data has to be committed:

      >>> transaction.commit()

    Let's now search and receive documents as result:

      >>> res = dm.root['people'].raw_find({'name': {'$regex': '^Ro.*'}})
      >>> pprint(list(res))
      [{u'_id': ObjectId('4e7eb152e138234158000004'),
        u'key': u'roy',
        u'name': u'Roy',
        u'parent': DBRef(u'mongopersist.zope.container.MongoContainer',
                         ObjectId('4e7eb152e138234158000000'),
                         u'mongopersist_container_test')},
       {u'_id': ObjectId('4e7eb152e138234158000005'),
        u'key': u'roger',
        u'name': u'Roger',
        u'parent': DBRef(u'mongopersist.zope.container.MongoContainer',
                         ObjectId('4e7eb152e138234158000000'),
                         u'mongopersist_container_test')}]

    And now the same query, but this time with object results:

      >>> res = dm.root['people'].find({'name': {'$regex': '^Ro.*'}})
      >>> pprint(list(res))
      [<Person Roy>, <Person Roger>]

    When no spec is specified, all items are returned:

      >>> res = dm.root['people'].find()
      >>> pprint(list(res))
      [<Person Stephan>, <Person Roy>, <Person Roger>, <Person Adam>,
       <Person Albertas>, <Person Russ>]

    You can also search for a single result:

      >>> res = dm.root['people'].raw_find_one({'name': {'$regex': '^St.*'}})
      >>> pprint(res)
      {u'_id': ObjectId('4e7eb259e138234289000003'),
       u'key': u'stephan',
       u'name': u'Stephan',
       u'parent': DBRef(u'mongopersist.zope.container.MongoContainer',
                        ObjectId('4e7eb259e138234289000000'),
                        u'mongopersist_container_test')}

      >>> stephan = dm.root['people'].find_one({'name': {'$regex': '^St.*'}})
      >>> pprint(stephan)
      <Person Stephan>

    If no result is found, ``None`` is returned:

      >>> dm.root['people'].find_one({'name': {'$regex': '^XXX.*'}})

    If there is no spec, then simply the first item is returned:

      >>> dm.root['people'].find_one()
      <Person Stephan>

    On the other hand, if the spec is an id, we look for it instead:

      >>> dm.root['people'].find_one(stephan._p_oid.id)
      <Person Stephan>
    """

def doctest_AllItemsMongoContainer_basic():
    """AllItemsMongoContainer: basic

    This type of container returns all items of the collection without regard
    of a parenting hierarchy.

    Let's start by creating two person containers that service different
    purposes:

      >>> transaction.commit()

      >>> dm.root['friends'] = container.MongoContainer('person')
      >>> dm.root['friends'][u'roy'] = Person(u'Roy')
      >>> dm.root['friends'][u'roger'] = Person(u'Roger')

      >>> dm.root['family'] = container.MongoContainer('person')
      >>> dm.root['family'][u'anton'] = Person(u'Anton')
      >>> dm.root['family'][u'konrad'] = Person(u'Konrad')

      >>> transaction.commit()
      >>> sorted(dm.root['friends'].keys())
      [u'roger', u'roy']
      >>> sorted(dm.root['family'].keys())
      [u'anton', u'konrad']

    Now we can create an all-items-container that allows us to view all
    people.

      >>> dm.root['all-people'] = container.AllItemsMongoContainer('person')
      >>> sorted(dm.root['all-people'].keys())
      [u'anton', u'konrad', u'roger', u'roy']
    """

def doctest_SubDocumentMongoContainer_basic():
    r"""SubDocumentMongoContainer: basic

    Let's make sure events are fired correctly:

      >>> zope.component.provideHandler(handleObjectModifiedEvent)

    Sub_document Mongo containers are useful, since they avoid the creation of
    a commonly trivial collections holding meta-data for the collection
    object. But they require a root document:

      >>> dm.reset()
      >>> dm.root['app_root'] = ApplicationRoot()

    Let's add a container to the app root:

      >>> dm.root['app_root']['people'] = \
      ...     container.SubDocumentMongoContainer('person')
      ContainerModifiedEvent: <ApplicationRoot>

      >>> transaction.commit()
      >>> db = dm._conn[DBNAME]
      >>> pprint(list(db['root'].find()))
      [{u'_id': ObjectId('4e7ea67be138233711000001'),
        u'data':
         {u'people':
          {u'_m_collection': u'person',
           u'_py_persistent_type':
               u'mongopersist.zope.container.SubDocumentMongoContainer'}}}]

    It is unfortunate that the '_m_collection' attribute is set. This is
    avoidable using a sub-class. Let's make sure the container can be loaded
    correctly:

      >>> dm.root['app_root']['people']
      <mongopersist.zope.container.SubDocumentMongoContainer ...>
      >>> dm.root['app_root']['people'].__parent__
      <ApplicationRoot>
      >>> dm.root['app_root']['people'].__name__
      'people'

    Let's add an item to the container:

      >>> dm.root['app_root']['people'][u'stephan'] = Person(u'Stephan')
      ContainerModifiedEvent: <...SubDocumentMongoContainer ...>
      >>> dm.root['app_root']['people'].keys()
      [u'stephan']
      >>> dm.root['app_root']['people'][u'stephan']
      <Person Stephan>

      >>> transaction.commit()
      >>> dm.root['app_root']['people'].keys()
      [u'stephan']
    """

def doctest_MongoContainer_with_ZODB():
    r"""MongoContainer: with ZODB

    This test demonstrates how a Mongo Container lives inside a ZODB tree:

      >>> zodb = ZODB.DB(ZODB.DemoStorage.DemoStorage())
      >>> root = zodb.open().root()
      >>> root['app'] = btree.BTreeContainer()
      >>> root['app']['people'] = container.MongoContainer('person')

    Let's now commit the transaction and make sure everything is cool.

      >>> transaction.commit()
      >>> root = zodb.open().root()
      >>> root['app']
      <zope.container.btree.BTreeContainer object at 0x7fbb5842f578>
      >>> root['app']['people']
      <mongopersist.zope.container.MongoContainer object at 0x7fd6e23555f0>

    Trying accessing people fails:

      >>> root['app']['people'].keys()
      Traceback (most recent call last):
      ...
      ComponentLookupError:
       (<InterfaceClass mongopersist.interfaces.IMongoDataManagerProvider>, '')

    This is because we have not told the system how to get a datamanager:

      >>> class Provider(object):
      ...     zope.interface.implements(interfaces.IMongoDataManagerProvider)
      ...     def get(self):
      ...         return dm
      >>> zope.component.provideUtility(Provider())

    So let's try again:

      >>> root['app']['people'].keys()
      []

    Next we create a person object and make sure it gets properly persisted.

      >>> root['app']['people']['stephan'] = Person(u'Stephan')
      >>> transaction.commit()
      >>> root = zodb.open().root()
      >>> root['app']['people'].keys()
      [u'stephan']

      >>> stephan = root['app']['people']['stephan']
      >>> stephan.__name__
      'stephan'
      >>> stephan.__parent__
      <mongopersist.zope.container.MongoContainer object at 0x7f6b6273b7d0>

      >>> pprint(list(dm._conn[DBNAME]['person'].find()))
      [{u'_id': ObjectId('4e7ed795e1382366a0000001'),
        u'key': u'stephan',
        u'name': u'Stephan',
        u'parent': u'zodb-1058e89d27d8afd9'}]

    Note that we produced a nice hex-presentation of the ZODB's OID.
    """

checker = renormalizing.RENormalizing([
    (re.compile(r'datetime.datetime(.*)'),
     'datetime.datetime(2011, 10, 1, 9, 45)'),
    (re.compile(r"ObjectId\('[0-9a-f]*'\)"),
     "ObjectId('4e7ddf12e138237403000000')"),
    (re.compile(r"object at 0x[0-9a-f]*>"),
     "object at 0x001122>"),
    (re.compile(r"zodb-[0-9a-f].*"),
     "zodb-01af3b00c5"),
    ])

@zope.component.adapter(
    zope.interface.Interface,
    zope.lifecycleevent.interfaces.IObjectModifiedEvent
    )
def handleObjectModifiedEvent(object, event):
    print event.__class__.__name__+':', repr(object)


def setUp(test):
    placelesssetup.setUp(test)
    module.setUp(test)
    test.globs['conn'] = pymongo.Connection('localhost', 27017, tz_aware=False)
    test.globs['DBNAME'] = 'mongopersist_container_test'
    test.globs['conn'].drop_database(test.globs['DBNAME'])
    test.globs['dm'] = datamanager.MongoDataManager(
        test.globs['conn'],
        default_database=test.globs['DBNAME'],
        root_database=test.globs['DBNAME'])

def tearDown(test):
    placelesssetup.tearDown(test)
    module.tearDown(test)
    test.globs['conn'].disconnect()
    serialize.SERIALIZERS.__init__()

def test_suite():
    return doctest.DocTestSuite(
        setUp=setUp, tearDown=tearDown, checker=checker,
        optionflags=(doctest.NORMALIZE_WHITESPACE|
                     doctest.ELLIPSIS|
                     doctest.REPORT_ONLY_FIRST_FAILURE
                     #|doctest.REPORT_NDIFF
                     )
        )



##############################################################################
#
# Copyright (c) 2011 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Mongo Persistent Data Manager"""
from __future__ import absolute_import
import UserDict
import persistent
import pymongo
import pymongo.dbref
import transaction
import zope.interface

from mongopersist import interfaces, serialize

def create_conflict_error(obj, new_doc):
    return interfaces.ConflictError(
        None, obj,
        (new_doc.get('_py_serial', 0), serialize.u64(obj._p_serial)))

def processSpec(collection, spec):
    try:
        adapter = interfaces.IMongoSpecProcessor(None)
    except TypeError:
        # by default nothing is registered, handle that case
        return spec

    return adapter.process(collection, spec)

class Root(UserDict.DictMixin):

    database='mongopersist'
    collection = 'persistence_root'

    def __init__(self, jar, database=None, collection=None):
        self._jar = jar
        if database is not None:
            self.database = database
        if collection is not None:
            self.collection = collection
        db = self._jar._conn[self.database]
        self._collection_inst = db[self.collection]

    def __getitem__(self, key):
        doc = self._collection_inst.find_one(
            processSpec(self._collection_inst, {'name': key}))
        if doc is None:
            raise KeyError(key)
        return self._jar.load(doc['ref'])

    def __setitem__(self, key, value):
        dbref = self._jar.dump(value)
        if self.get(key) is not None:
            del self[key]
        doc = {'ref': dbref, 'name': key}
        self._collection_inst.insert(doc)

    def __delitem__(self, key):
        doc = self._collection_inst.find_one(
            processSpec(self._collection_inst, {'name': key}))
        coll = self._jar._conn[doc['ref'].database][doc['ref'].collection]
        coll.remove(doc['ref'].id)
        self._collection_inst.remove({'name': key})

    def keys(self):
        return [doc['name'] for doc in self._collection_inst.find()]


class MongoDataManager(object):
    zope.interface.implements(interfaces.IMongoDataManager)

    detect_conflicts = False
    default_database = 'mongopersist'
    name_map_collection = 'persistence_name_map'
    conflict_error_factory = staticmethod(create_conflict_error)

    def __init__(self, conn, detect_conflicts=None, default_database=None,
                 root_database=None, root_collection=None,
                 name_map_collection=None, conflict_error_factory=None):
        self._conn = conn
        self._reader = serialize.ObjectReader(self)
        self._writer = serialize.ObjectWriter(self)
        self._registered_objects = []
        self._loaded_objects = []
        self._needs_to_join = True
        self._object_cache = {}
        self.annotations = {}
        if detect_conflicts is not None:
            self.detect_conflicts = detect_conflicts
        if default_database is not None:
            self.default_database = default_database
        if name_map_collection is not None:
            self.name_map_collection = name_map_collection
        if conflict_error_factory is not None:
            self.conflict_error_factory = conflict_error_factory
        self.transaction_manager = transaction.manager
        self.root = Root(self, root_database, root_collection)

    def dump(self, obj):
        return self._writer.store(obj)

    def load(self, dbref):
        return self._reader.get_ghost(dbref)

    def reset(self):
        root = self.root
        self.__init__(self._conn)
        self.root = root

    def setstate(self, obj):
        # When reading a state from Mongo, we also need to join the
        # transaction, because we keep an active object cache that gets stale
        # after the transaction is complete and must be cleaned.
        if self._needs_to_join:
            self.transaction_manager.get().join(self)
            self._needs_to_join = False
        self._reader.set_ghost_state(obj)
        self._loaded_objects.append(obj)

    def oldstate(self, obj, tid):
        # I cannot find any code using this method. Also, since we do not keep
        # version history, we always raise an error.
        raise KeyError(tid)

    def register(self, obj):
        if self._needs_to_join:
            self.transaction_manager.get().join(self)
            self._needs_to_join = False

        if obj is not None and obj not in self._registered_objects:
            self._registered_objects.append(obj)

    def abort(self, transaction):
        self.reset()

    def commit(self, transaction):
        if not self.detect_conflicts:
            return
        # Check each modified object to see whether Mongo has a new version of
        # the object.
        for obj in self._registered_objects:
            # This object is not even added to the database yet, so there
            # cannot be a conflict.
            if obj._p_oid is None:
                continue
            db_name, coll_name = self._writer.get_collection_name(obj)
            coll = self._conn[db_name][coll_name]
            new_doc = coll.find_one(obj._p_oid.id, fields=('_py_serial',))
            if new_doc is None:
                continue
            if new_doc.get('_py_serial', 0) != serialize.u64(obj._p_serial):
                raise self.conflict_error_factory(obj, new_doc)

    def tpc_begin(self, transaction):
        pass

    def tpc_vote(self, transaction):
        pass

    def tpc_finish(self, transaction):
        written = []
        for obj in self._registered_objects:
            if getattr(obj, '_p_mongo_sub_object', False):
                obj = obj._p_mongo_doc_object
            if obj in written:
                continue
            self._writer.store(obj)
            written.append(obj)
        self.reset()

    def tpc_abort(self, transaction):
        self.abort(transaction)

    def sortKey(self):
        return ('MongoDataManager', 0)



##############################################################################
#
# Copyright (c) 2011 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Mongo Mapping Implementations"""
import UserDict
import pymongo

from mongopersist import interfaces
from mongopersist.datamanager import processSpec

class MongoCollectionMapping(UserDict.DictMixin, object):
    __mongo_database__ = None
    __mongo_collection__ = None
    __mongo_mapping_key__ = 'key'

    def __init__(self, jar):
        self._m_jar = jar

    def __mongo_filter__(self):
        return {}

    def get_mongo_collection(self):
        db_name = self.__mongo_database__ or self._m_jar.default_database
        return self._m_jar._conn[db_name][self.__mongo_collection__]

    def __getitem__(self, key):
        filter = self.__mongo_filter__()
        filter[self.__mongo_mapping_key__] = key
        coll = self.get_mongo_collection()
        doc = coll.find_one(processSpec(coll, filter))
        if doc is None:
            raise KeyError(key)
        db_name = self.__mongo_database__ or self._m_jar.default_database
        dbref = pymongo.dbref.DBRef(
            self.__mongo_collection__, doc['_id'], db_name)
        return self._m_jar._reader.get_ghost(dbref)

    def __setitem__(self, key, value):
        # Even though setting the attribute should register the object with
        # the data manager, the value might not be in the DB at all at this
        # point, so registering it manually ensures that new objects get added.
        self._m_jar.register(value)
        setattr(value, self.__mongo_mapping_key__, key)

    def __delitem__(self, key):
        # Deleting the object from the database is not our job. We simply
        # remove it from the dictionary.
        value = self[key]
        setattr(value, self.__mongo_mapping_key__, None)

    def keys(self):
        filter = self.__mongo_filter__()
        filter[self.__mongo_mapping_key__] = {'$ne': None}
        coll = self.get_mongo_collection()
        return [
            doc[self.__mongo_mapping_key__]
            for doc in coll.find(processSpec(coll, filter))]



##############################################################################
#
# Copyright (c) 2011 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Mongo Persistence Serializeation Tests"""
import datetime
import doctest
import persistent
import pprint

from pymongo import binary, dbref, objectid

from mongopersist import testing, serialize

class Top(persistent.Persistent):
    _p_mongo_collection = 'Top'

def create_top(name):
    top = Top()
    top.name = name
    return top

class Top2(Top):
    pass

class Tier2(persistent.Persistent):
    _p_mongo_sub_object = True

class Foo(persistent.Persistent):
    _p_mongo_collection = 'Foo'

class Anything(persistent.Persistent):
    pass

class Simple(object):
    pass

def doctest_ObjectSerializer():
    """Test the abstract ObjectSerializer class.

    Object serializers are hooks into the serialization process to allow
    better serialization for particular objects. For example, the result of
    reducing a datetime.date object is a short, optimized binary string. This
    representation might be optimal for pickles, but is really aweful for
    Mongo, since it does not allow querying for dates. An object serializer
    can be used to use a better representation, such as the date ordinal
    number.

      >>> os = serialize.ObjectSerializer()

    So here are the methods that must be implemented by an object serializer:

      >>> os.can_read({})
      Traceback (most recent call last):
      ...
      NotImplementedError

      >>> os.read({})
      Traceback (most recent call last):
      ...
      NotImplementedError

      >>> os.can_write(object())
      Traceback (most recent call last):
      ...
      NotImplementedError

      >>> os.write(object())
      Traceback (most recent call last):
      ...
      NotImplementedError
    """

def doctest_ObjectWriter_get_collection_name():
    """ObjectWriter: get_collection_name()

    This method determines the collection name and database for a given
    object. It can either be specified via '_p_mongo_collection' or is
    determined from the class path. When the collection name is specified, the
    mapping from collection name to class path is stored.

      >>> print tuple(conn[DBNAME][dm.name_map_collection].find())
      ()

      >>> writer = serialize.ObjectWriter(dm)
      >>> writer.get_collection_name(Anything())
      ('mongopersist_test', 'mongopersist.tests.test_serialize.Anything')

      >>> top = Top()
      >>> writer.get_collection_name(top)
      ('mongopersist_test', 'Top')

      >>> print tuple(conn[DBNAME][dm.name_map_collection].find())
      ({u'path': u'mongopersist.tests.test_serialize.Top',
        u'doc_has_type': False,
        u'_id': ObjectId('4eb19f9937a08e27b7000000'),
        u'collection': u'Top',
        u'database': u'mongopersist_test'},)

      >>> getattr(top, '_p_mongo_store_type', None)

    When classes use inheritance, it often happens that all sub-objects share
    the same collection. However, only one can have an entry in our mapping
    table to avoid non-unique answers. Thus we require all sub-types after the
    first one to store their typing providing a hint for deseriealization:

      >>> top2 = Top2()
      >>> writer.get_collection_name(top2)
      ('mongopersist_test', 'Top')

      >>> pprint.pprint(tuple(conn[DBNAME][dm.name_map_collection].find()))
      ({u'_id': ObjectId('4eb1b5ab37a08e2f06000000'),
        u'collection': u'Top',
        u'database': u'mongopersist_test',
        u'doc_has_type': False,
        u'path': u'mongopersist.tests.test_serialize.Top'},
       {u'_id': ObjectId('4eb1b5ab37a08e2f06000001'),
        u'collection': u'Top',
        u'database': u'mongopersist_test',
        u'doc_has_type': True,
        u'path': u'mongopersist.tests.test_serialize.Top2'})

      >>> getattr(top2, '_p_mongo_store_type', None)
      True
    """

def doctest_ObjectWriter_get_non_persistent_state():
    r"""ObjectWriter: get_non_persistent_state()

    This method produces a proper reduced state for custom, non-persistent
    objects.

      >>> writer = serialize.ObjectWriter(dm)

    A simple new-style class:

      >>> class This(object):
      ...     def __init__(self, num):
      ...         self.num = num

      >>> this = This(1)
      >>> writer.get_non_persistent_state(this, [])
      {'num': 1, '_py_type': '__main__.This'}

    A simple old-style class:

      >>> class That(object):
      ...     def __init__(self, num):
      ...         self.num = num

      >>> that = That(1)
      >>> writer.get_non_persistent_state(that, [])
      {'num': 1, '_py_type': '__main__.That'}

    The method also handles persistent classes that do not want their own
    document:

      >>> top = Top()
      >>> writer.get_non_persistent_state(top, [])
      {'_py_persistent_type': 'mongopersist.tests.test_serialize.Top'}

    And then there are the really weird cases:

      >>> writer.get_non_persistent_state(datetime.date(2011, 11, 1), [])
      {'_py_factory': 'datetime.date',
       '_py_factory_args': [Binary('\x07\xdb\x0b\x01', 0)]}

    Circular object references cause an error:

      >>> writer.get_non_persistent_state(this, [this])
      Traceback (most recent call last):
      ...
      CircularReferenceError: <__main__.This object at 0x3051550>
    """

def doctest_ObjectWriter_get_persistent_state():
    r"""ObjectWriter: get_persistent_state()

    This method produces a proper reduced state for a persistent object, which
    is basically a Mongo DBRef.

      >>> writer = serialize.ObjectWriter(dm)

      >>> foo = Foo()
      >>> foo._p_oid
      >>> list(conn[DBNAME]['Foo'].find())
      []

      >>> writer.get_persistent_state(foo, [])
      DBRef('Foo', ObjectId('4eb1a87f37a08e29ff000002'), 'mongopersist_test')

      >>> foo._p_oid
      DBRef('Foo', ObjectId('4eb1a87f37a08e29ff000002'), 'mongopersist_test')
      >>> pprint.pprint(list(conn[DBNAME]['Foo'].find()))
      [{u'_id': ObjectId('4eb1a96c37a08e2a7b000002')}]

    The next time the object simply returns its reference:

      >>> writer.get_persistent_state(foo, [])
      DBRef('Foo', ObjectId('4eb1a87f37a08e29ff000002'), 'mongopersist_test')
      >>> pprint.pprint(list(conn[DBNAME]['Foo'].find()))
      [{u'_id': ObjectId('4eb1a96c37a08e2a7b000002')}]
    """


def doctest_ObjectWriter_get_state_MONGO_NATIVE_TYPES():
    """ObjectWriter: get_state(): Mongo-native Types

      >>> writer = serialize.ObjectWriter(None)
      >>> writer.get_state(1)
      1
      >>> writer.get_state(1.0)
      1.0
      >>> writer.get_state(u'Test')
      u'Test'
      >>> writer.get_state(datetime.datetime(2011, 11, 1, 12, 0, 0))
      datetime.datetime(2011, 11, 1, 12, 0, 0)
      >>> print writer.get_state(None)
      None
      >>> writer.get_state(objectid.ObjectId('4e7ddf12e138237403000000'))
      ObjectId('4e7ddf12e138237403000000')
      >>> writer.get_state(dbref.DBRef('4e7ddf12e138237403000000', 'test'))
      DBRef('4e7ddf12e138237403000000', 'test')
    """

def doctest_ObjectWriter_get_state_types():
    """ObjectWriter: get_state(): types (type, class)

      >>> writer = serialize.ObjectWriter(None)
      >>> writer.get_state(Top)
      {'path': 'mongopersist.tests.test_serialize.Top', '_py_type': 'type'}
      >>> writer.get_state(str)
      {'path': '__builtin__.str', '_py_type': 'type'}
    """

def doctest_ObjectWriter_get_state_sequences():
    """ObjectWriter: get_state(): sequences (tuple, list, PersistentList)

    We convert any sequence into a simple list, since Mongo supports that
    type natively. But also reduce any sub-objects.

      >>> class Number(object):
      ...     def __init__(self, num):
      ...         self.num = num

      >>> writer = serialize.ObjectWriter(None)
      >>> writer.get_state((1, '2', Number(3)))
      [1, '2', {'num': 3, '_py_type': '__main__.Number'}]
      >>> writer.get_state([1, '2', Number(3)])
      [1, '2', {'num': 3, '_py_type': '__main__.Number'}]
    """

def doctest_ObjectWriter_get_state_mappings():
    """ObjectWriter: get_state(): mappings (dict, PersistentDict)

    We convert any mapping into a simple dict, since Mongo supports that
    type natively. But also reduce any sub-objects.

      >>> class Number(object):
      ...     def __init__(self, num):
      ...         self.num = num

      >>> writer = serialize.ObjectWriter(None)
      >>> writer.get_state({'1': 1, '2': '2', '3': Number(3)})
      {'1': 1, '3': {'num': 3, '_py_type': '__main__.Number'}, '2': '2'}

    Unfortunately, Mongo only supports text keys. So whenever we have non-text
    keys, we need to create a less natural, but consistent structure:

      >>> writer.get_state({1: 'one', 2: 'two', 3: 'three'})
      {'dict_data': [(1, 'one'), (2, 'two'), (3, 'three')]}
    """

def doctest_ObjectWriter_get_state_Persistent():
    """ObjectWriter: get_state(): Persistent objects

      >>> writer = serialize.ObjectWriter(dm)

      >>> top = Top()
      >>> writer.get_state(top)
      DBRef('Top', ObjectId('4eb1aede37a08e2c8d000004'), 'mongopersist_test')

    But a persistent object can declare that it does not want a separate
    document:

      >>> top2 = Top()
      >>> top2._p_mongo_sub_object = True
      >>> writer.get_state(top2)
      {'_py_persistent_type': 'mongopersist.tests.test_serialize.Top'}
    """

def doctest_ObjectWriter_store():
    """ObjectWriter: store()

      >>> writer = serialize.ObjectWriter(dm)

    Simply store an object:

      >>> pprint.pprint(list(conn[DBNAME]['Top'].find()))
      []

      >>> top = Top()
      >>> writer.store(top)
      DBRef('Top', ObjectId('4eb1b16537a08e2d1a000001'), 'mongopersist_test')
      >>> pprint.pprint(list(conn[DBNAME]['Top'].find()))
      [{u'_id': ObjectId('4eb1b17937a08e2d29000001')}]

    Now that we have an object, storing an object simply means updating the
    existing document:

      >>> top.name = 'top'
      >>> writer.store(top)
      DBRef('Top', ObjectId('4eb1b16537a08e2d1a000001'), 'mongopersist_test')
      >>> pprint.pprint(list(conn[DBNAME]['Top'].find()))
      [{u'_id': ObjectId('4eb1b17937a08e2d29000001'), u'name': u'top'}]

    """

def doctest_ObjectWriter_store_with_mongo_store_type():
    """ObjectWriter: store(): _p_mongo_store_type = True

      >>> writer = serialize.ObjectWriter(dm)

      >>> top = Top()
      >>> top._p_mongo_store_type = True
      >>> writer.store(top)
      DBRef('Top', ObjectId('4eb1b16537a08e2d1a000001'), 'mongopersist_test')
      >>> pprint.pprint(list(conn[DBNAME]['Top'].find()))
      [{u'_id': ObjectId('4eb1b27437a08e2d7d000003'),
        u'_py_persistent_type': u'mongopersist.tests.test_serialize.Top'}]
    """

def doctest_ObjectWriter_store_with_conflict_detection():
    """ObjectWriter: store(): conflict detection

    The writer supports the data manager's conflict detection by storing a
    serial number, which is effectively the version of the object. The data
    manager can then use the serial to detect whether a competing transaction
    has written to the document.

      >>> dm.detect_conflicts = True
      >>> writer = serialize.ObjectWriter(dm)

      >>> top = Top()
      >>> writer.store(top)
      DBRef('Top', ObjectId('4eb1b16537a08e2d1a000001'), 'mongopersist_test')
      >>> pprint.pprint(list(conn[DBNAME]['Top'].find()))
      [{u'_id': ObjectId('4eb1b31137a08e2d9d000003'), u'_py_serial': 1}]
    """

def doctest_ObjectWriter_store_with_new_object_references():
    """ObjectWriter: store(): new object references

    When two new objects reference each other, extracting the full state would
    cause infinite recursion errors. The code protects against that by
    optionally only creating an initial empty reference document.

      >>> writer = serialize.ObjectWriter(dm)

      >>> top = Top()
      >>> top.foo = Foo()
      >>> top.foo.top = top
      >>> writer.store(top)
      DBRef('Top', ObjectId('4eb1b16537a08e2d1a000001'), 'mongopersist_test')
      >>> pprint.pprint(list(conn[DBNAME]['Top'].find()))
      [{u'_id': ObjectId('4eb1b3d337a08e2de7000009'),
        u'foo': DBRef(u'Foo', ObjectId('4eb1b3d337a08e2de7000008'),
                      u'mongopersist_test')}]
    """

def doctest_ObjectReader_simple_resolve():
    """ObjectReader: simple_resolve()

    This methods simply resolves a Python path to the represented object.

      >>> reader = serialize.ObjectReader(dm)
      >>> reader.simple_resolve('mongopersist.tests.test_serialize.Top')
      <class 'mongopersist.tests.test_serialize.Top'>
    """

def doctest_ObjectReader_resolve_simple():
    """ObjectReader: resolve(): simple

    This methods resolves a collection name to its class. The collection name
    can be either any arbitrary string or a Python path.

      >>> reader = serialize.ObjectReader(dm)
      >>> ref = dbref.DBRef('mongopersist.tests.test_serialize.Top',
      ...                   '4eb1b3d337a08e2de7000100')
      >>> reader.resolve(ref)
      <class 'mongopersist.tests.test_serialize.Top'>
    """

def doctest_ObjectReader_resolve_lookup():
    """ObjectReader: resolve(): lookup

    If Python path resolution fails, we try to lookup the path from the
    collection mapping collection names to Python paths.

      >>> reader = serialize.ObjectReader(dm)
      >>> ref = dbref.DBRef('Top', '4eb1b3d337a08e2de7000100', DBNAME)
      >>> reader.resolve(ref)
      Traceback (most recent call last):
      ...
      ImportError: DBRef('Top', '4eb1b3d337a08e2de7000100', 'mongopersist_test')

    The lookup failed, because there is no map entry yet for the 'Top'
    collection. The easiest way to create one is with the object writer:

      >>> top = Top()
      >>> writer = serialize.ObjectWriter(dm)
      >>> writer.get_collection_name(top)
      ('mongopersist_test', 'Top')

      >>> reader.resolve(ref)
      <class 'mongopersist.tests.test_serialize.Top'>
    """

def doctest_ObjectReader_resolve_lookup_with_multiple_maps():
    """ObjectReader: resolve(): lookup with multiple maps entries

    When the collection name to Python path map has multiple entries, things
    are more interesting. In this case, we need to lookup the object, if it
    stores its persistent type otherwise we use the first map entry.

      >>> writer = serialize.ObjectWriter(dm)
      >>> top = Top()
      >>> writer.store(top)
      DBRef('Top', ObjectId('4eb1e0f237a08e38dd000002'), 'mongopersist_test')
      >>> top2 = Top2()
      >>> writer.store(top2)
      DBRef('Top', ObjectId('4eb1e10437a08e38e8000004'), 'mongopersist_test')

      >>> reader = serialize.ObjectReader(dm)
      >>> reader.resolve(top._p_oid)
      <class 'mongopersist.tests.test_serialize.Top'>
      >>> reader.resolve(top2._p_oid)
      <class 'mongopersist.tests.test_serialize.Top2'>

      >>> pprint.pprint(list(conn[DBNAME]['Top'].find()))
      [{u'_id': ObjectId('4eb1e13337a08e392d000002')},
       {u'_id': ObjectId('4eb1e13337a08e392d000004'),
        u'_py_persistent_type': u'mongopersist.tests.test_serialize.Top2'}]

    If the DBRef does not have an object id, then an import error is raised:

      >>> reader.resolve(dbref.DBRef('Top', None, 'mongopersist_test'))
      Traceback (most recent call last):
      ...
      ImportError: DBRef('Top', None, 'mongopersist_test')
    """

def doctest_ObjectReader_get_non_persistent_object_py_type():
    """ObjectReader: get_non_persistent_object(): _py_type

    The simplest case is a document with a _py_type:

      >>> reader = serialize.ObjectReader(dm)
      >>> reader.get_non_persistent_object(
      ...    {'_py_type': 'mongopersist.tests.test_serialize.Simple'}, None)
      <mongopersist.tests.test_serialize.Simple object at 0x306f410>

    It is a little bit more interesting when there is some additional state:

      >>> simple = reader.get_non_persistent_object(
      ...    {u'_py_type': 'mongopersist.tests.test_serialize.Simple',
      ...     u'name': u'Here'},
      ...    None)
      >>> simple.name
      u'Here'
    """

def doctest_ObjectReader_get_non_persistent_object_py_persistent_type():
    """ObjectReader: get_non_persistent_object(): _py_persistent_type

    In this case the document has a _py_persistent_type attribute, which
    signals a persistent object living in its parent's document:

      >>> top = Top()

      >>> reader = serialize.ObjectReader(dm)
      >>> tier2 = reader.get_non_persistent_object(
      ...    {'_py_persistent_type': 'mongopersist.tests.test_serialize.Tier2',
      ...     'name': 'Number 2'},
      ...    top)
      >>> tier2
      <mongopersist.tests.test_serialize.Tier2 object at 0x306f410>

    We keep track of the containing object, so we can set _p_changed when this
    object changes.

      >>> tier2._p_mongo_doc_object
      <mongopersist.tests.test_serialize.Top object at 0x7fa30b534050>
      >>> tier2._p_jar
      <mongopersist.datamanager.MongoDataManager object at 0x7fc3cab375d0>
    """

def doctest_ObjectReader_get_non_persistent_object_py_factory():
    """ObjectReader: get_non_persistent_object(): _py_factory

    This is the case of last resort. Specify a factory and its arguments:

      >>> reader = serialize.ObjectReader(dm)
      >>> top = reader.get_non_persistent_object(
      ...    {'_py_factory': 'mongopersist.tests.test_serialize.create_top',
      ...     '_py_factory_args': ('TOP',)},
      ...    None)
      >>> top
      <mongopersist.tests.test_serialize.Top object at 0x306f410>
      >>> top.name
      'TOP'
    """

def doctest_ObjectReader_get_object_ObjectId():
    """ObjectReader: get_object(): ObjectId

    The object id is special and we simply conserve it:

      >>> reader = serialize.ObjectReader(dm)
      >>> reader.get_object(
      ...     objectid.ObjectId('4e827608e13823598d000003'), None)
      ObjectId('4e827608e13823598d000003')
    """

def doctest_ObjectReader_get_object_binary():
    """ObjectReader: get_object(): binary data

    Binary data is just converted to a string:

      >>> reader = serialize.ObjectReader(dm)
      >>> reader.get_object(binary.Binary('hello'), None)
      'hello'
    """

def doctest_ObjectReader_get_object_dbref():
    """ObjectReader: get_object(): DBRef

      >>> writer = serialize.ObjectWriter(dm)
      >>> top = Top()
      >>> writer.store(top)
      DBRef('Top', ObjectId('4eb1e0f237a08e38dd000002'), 'mongopersist_test')

    Database references load the ghost state of the obejct they represent:

      >>> reader = serialize.ObjectReader(dm)
      >>> reader.get_object(top._p_oid, None)
      <mongopersist.tests.test_serialize.Top object at 0x2801938>
    """

def doctest_ObjectReader_get_object_type_ref():
    """ObjectReader: get_object(): type reference

    Type references are resolved.

      >>> reader = serialize.ObjectReader(dm)
      >>> reader.get_object(
      ...     {'_py_type': 'type',
      ...      'path': 'mongopersist.tests.test_serialize.Simple'},
      ...     None)
      <class 'mongopersist.tests.test_serialize.Simple'>
    """

def doctest_ObjectReader_get_object_instance():
    """ObjectReader: get_object(): instance

    Instances are completely loaded:

      >>> reader = serialize.ObjectReader(dm)
      >>> simple = reader.get_object(
      ...     {u'_py_type': 'mongopersist.tests.test_serialize.Simple',
      ...      u'name': u'easy'},
      ...     None)
      >>> simple
      <mongopersist.tests.test_serialize.Simple object at 0x2bcc950>
      >>> simple.name
      u'easy'
    """

def doctest_ObjectReader_get_object_sequence():
    """ObjectReader: get_object(): sequence

    Sequences become persistent lists with all obejcts deserialized.

      >>> reader = serialize.ObjectReader(dm)
      >>> reader.get_object([1, '2', 3.0], None)
      [1, '2', 3.0]
    """

def doctest_ObjectReader_get_object_mapping():
    """ObjectReader: get_object(): mapping

    Mappings become persistent dicts with all obejcts deserialized.

      >>> reader = serialize.ObjectReader(dm)
      >>> pprint.pprint(reader.get_object({'1': 1, '2': 2, '3': 3}, None))
      {'1': 1, '3': 3, '2': 2}

    Since Mongo does not allow for non-string keys, the state for a dict with
    non-string keys looks different:

      >>> pprint.pprint(reader.get_object(
      ...     {'dict_data': [(1, '1'), (2, '2'), (3, '3')]},
      ...     None))
      {1: '1', 2: '2', 3: '3'}
    """

def doctest_ObjectReader_get_ghost():
    """ObjectReader: get_ghost()

      >>> writer = serialize.ObjectWriter(dm)
      >>> top = Top()
      >>> writer.store(top)
      DBRef('Top', ObjectId('4eb1e0f237a08e38dd000002'), 'mongopersist_test')

    The ghost object is a shell without any loaded object state:

      >>> reader = serialize.ObjectReader(dm)
      >>> gobj = reader.get_ghost(top._p_oid)
      >>> gobj._p_jar
      <mongopersist.datamanager.MongoDataManager object at 0x2720e50>
      >>> gobj._p_state
      0

    The second time we look up the object, it comes from cache:

      >>> gobj = reader.get_ghost(top._p_oid)
      >>> gobj._p_state
      0
    """

def doctest_ObjectReader_set_ghost_state():
    r"""ObjectReader: set_ghost_state()

      >>> dm.detect_conflicts = True

      >>> writer = serialize.ObjectWriter(dm)
      >>> top = Top()
      >>> top.name = 'top'
      >>> writer.store(top)
      DBRef('Top', ObjectId('4eb1e0f237a08e38dd000002'), 'mongopersist_test')

    The ghost object is a shell without any loaded object state:

      >>> reader = serialize.ObjectReader(dm)
      >>> gobj = reader.get_ghost(top._p_oid)
      >>> gobj._p_jar
      <mongopersist.datamanager.MongoDataManager object at 0x2720e50>
      >>> gobj._p_state
      0

    Now load the state:

      >>> reader.set_ghost_state(gobj)
      >>> gobj.name
      u'top'
      >>> gobj._p_serial
      '\x00\x00\x00\x00\x00\x00\x00\x01'
    """



def doctest_deserialize_persistent_references():
    """Deserialization o persistent references.

    The purpose of this test is to demonstrate the proper deserialization of
    persistent object references.

    Let's create a simple object hierarchy:

      >>> top = Top()
      >>> top.name = 'top'
      >>> top.foo = Foo()
      >>> top.foo.name = 'foo'

      >>> dm.root['top'] = top
      >>> commit()

    Let's check that the objects were properly serialized.

      >>> pprint.pprint(list(conn[DBNAME]['Top'].find()))
      [{u'_id': ObjectId('4e827608e13823598d000003'),
        u'foo': DBRef(u'Foo',
                      ObjectId('4e827608e13823598d000002'),
                      u'mongopersist_test'),
        u'name': u'top'}]
      >>> pprint.pprint(list(conn[DBNAME]['Foo'].find()))
      [{u'_id': ObjectId('4e8276c3e138235a2e000002'), u'name': u'foo'}]

    Now we access the objects objects again to see whether they got properly
    deserialized.

      >>> top2 = dm.root['top']
      >>> id(top2) == id(top)
      False
      >>> top2.name
      u'top'

      >>> id(top2.foo) == id(top.foo)
      False
      >>> top2.foo
      <mongopersist.tests.test_serialize.Foo object at 0x7fb1a0c0b668>
      >>> top2.foo.name
      u'foo'
    """


def test_suite():
    return doctest.DocTestSuite(
        setUp=testing.setUp, tearDown=testing.tearDown,
        checker=testing.checker,
        optionflags=testing.OPTIONFLAGS)



##############################################################################
#
# Copyright (c) 2011 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Mongo Persistence Zope Container Interfaces"""
import zope.interface
import zope.schema


class IMongoContainer(zope.interface.Interface):
    _m_database = zope.schema.ASCIILine(
        title=u'Mongo Database',
        description=(
            u'Specifies the MDB in which to store items. If ``None``, the '
            u'default database will be used.'),
        default=None)

    _m_collection = zope.schema.ASCIILine(
        title=u'Mongo Collection',
        description=(
            u'Specifies the Mongo collection in which to store items.')
        )

    _m_mapping_key = zope.schema.ASCIILine(
        title=u'Mapping Key',
        description=(
            u'Specifies the attribute name of the item that is used as the '
            u'mapping/dictionary/container key.'),
        default='key')

    _m_parent_key = zope.schema.ASCIILine(
        title=u'Parent Key',
        description=(
            u'Specifies the attribute name of the item that is used to store '
            u'the parent/container reference.'),
        default='parent')

    def _m_get_parent_key_value():
        """Returns the value that is used to specify a particular container as
        the parent of the item.
        """

    def _m_get_items_filter():
        """Returns a query spec representing a filter that only returns
        objects in this container."""

    def get_collection():
        """Get the Python representation of the collection.

        This can be useful to make custom queries against the collection.
        """

    def raw_find(spec=None, *args, **kwargs):
        """Return a raw Mongo result set for the specified query.

        The spec is updated to also contain the container's filter spec.

        See pymongo's documentation for details on *args and **kwargs.
        """

    def find(spec=None, fields=None, *args, **kwargs):
        """Return a Python object result set for the specified query.

        By default only the Mongo Id and key attribute is requested and a
        ghost is created. The rest of the data is only retrieved if needed.

        The spec is updated to also contain the container's filter spec.

        See pymongo's documentation for details on *args and **kwargs.
        """

    def raw_find_one(spec_or_id=None, *args, **kwargs):
        """Return a raw Mongo document for the specified query.

        The spec is updated to also contain the container's filter spec.

        See pymongo's documentation for details on *args and **kwargs.
        """

    def find_one(spec_or_id=None, fields=None, *args, **kwargs):
        """Return a single Python object for the specified query.

        The spec is updated to also contain the container's filter spec.

        See pymongo's documentation for details on *args and **kwargs.
        """



##############################################################################
#
# Copyright (c) 2011 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Mongo Persistence Testing Support"""
from __future__ import absolute_import
import doctest
import pymongo
import re
import transaction
from zope.testing import module, renormalizing

from mongopersist import datamanager, serialize

checker = renormalizing.RENormalizing([
    (re.compile(r'datetime.datetime(.*)'),
     'datetime.datetime(2011, 10, 1, 9, 45)'),
    (re.compile(r"ObjectId\('[0-9a-f]*'\)"),
     "ObjectId('4e7ddf12e138237403000000')"),
    (re.compile(r"object at 0x[0-9a-f]*>"),
     "object at 0x001122>"),
    ])

OPTIONFLAGS = (doctest.NORMALIZE_WHITESPACE|
               doctest.ELLIPSIS|
               doctest.REPORT_ONLY_FIRST_FAILURE
               #|doctest.REPORT_NDIFF
               )

def setUp(test):
    module.setUp(test)
    test.globs['conn'] = pymongo.Connection('localhost', 27017, tz_aware=False)
    test.globs['DBNAME'] = 'mongopersist_test'
    test.globs['conn'].drop_database(test.globs['DBNAME'])
    test.globs['commit'] = transaction.commit
    test.globs['dm'] = datamanager.MongoDataManager(
        test.globs['conn'],
        default_database=test.globs['DBNAME'],
        root_database=test.globs['DBNAME'])

def tearDown(test):
    module.tearDown(test)
    transaction.abort()
    test.globs['conn'].drop_database(test.globs['DBNAME'])
    test.globs['conn'].disconnect()
    serialize.SERIALIZERS.__init__()



##############################################################################
#
# Copyright (c) 2011 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Object Serialization for Mongo/BSON"""
from __future__ import absolute_import
import copy_reg
import struct

import lru
import persistent.interfaces
import persistent.dict
import persistent.list
import pymongo.binary
import pymongo.dbref
import pymongo.objectid
import types
import zope.interface
from zope.dottedname.resolve import resolve

from mongopersist import interfaces

SERIALIZERS = []
OID_CLASS_LRU = lru.LRUCache(20000)

def p64(v):
    """Pack an integer or long into a 8-byte string"""
    return struct.pack(">Q", v)

def u64(v):
    """Unpack an 8-byte string into a 64-bit long integer."""
    return struct.unpack(">Q", v)[0]

def get_dotted_name(obj):
    return obj.__module__+'.'+obj.__name__

class PersistentDict(persistent.dict.PersistentDict):
    _p_mongo_sub_object = True

class PersistentList(persistent.list.PersistentList):
    _p_mongo_sub_object = True


class ObjectSerializer(object):
    zope.interface.implements(interfaces.IObjectSerializer)

    def can_read(self, state):
        raise NotImplementedError

    def read(self, state):
        raise NotImplementedError

    def can_write(self, obj):
        raise NotImplementedError

    def write(self, obj):
        raise NotImplementedError


class ObjectWriter(object):
    zope.interface.implements(interfaces.IObjectWriter)

    def __init__(self, jar):
        self._jar = jar

    def get_collection_name(self, obj):
        db_name = getattr(obj, '_p_mongo_database', self._jar.default_database)
        try:
            coll_name = obj._p_mongo_collection
        except AttributeError:
            return db_name, get_dotted_name(obj.__class__)
        # Make sure that the coll_name to class path mapping is available.
        db = self._jar._conn[self._jar.default_database]
        coll = db[self._jar.name_map_collection]
        map = {'collection': coll_name,
               'database': db_name,
               'path': get_dotted_name(obj.__class__)}
        result = coll.find_one(map)
        if result is None:
            # If there is already a map for this collection, the next map must
            # force the object to store the type.
            result = coll.find({'collection': coll_name,
                                'database': db_name})
            if result.count() > 0:
                setattr(obj, '_p_mongo_store_type', True)
            map['doc_has_type'] = getattr(obj, '_p_mongo_store_type', False)
            coll.save(map)
        return db_name, coll_name

    def get_non_persistent_state(self, obj, seen):
        __traceback_info__ = obj
        # XXX: Look at the pickle library how to properly handle all types and
        # old-style classes with all of the possible pickle extensions.

        # Only non-persistent, custom objects can produce unresolvable
        # circular references.
        if obj in seen:
            raise interfaces.CircularReferenceError(obj)
        # Add the current object to the list of seen objects.
        seen.append(obj)
        # Get the state of the object. Only pickable objects can be reduced.
        reduced = obj.__reduce__()
        # The full object state (item 3) seems to be optional, so let's make
        # sure we handle that case gracefully.
        if len(reduced) == 2:
            factory, args = obj.__reduce__()
            obj_state = {}
        else:
            factory, args, obj_state = reduced
        # We are trying very hard to create a clean Mongo (sub-)document. But
        # we need a little bit of meta-data to help us out later.
        if factory == copy_reg._reconstructor and \
               args == (obj.__class__, object, None):
            # This is the simple case, which means we can produce a nicer
            # Mongo output.
            state = {'_py_type': get_dotted_name(args[0])}
        elif factory == copy_reg.__newobj__ and args == (obj.__class__,):
            # Another simple case for persistent objects that do not want
            # their own document.
            state = {'_py_persistent_type': get_dotted_name(args[0])}
        else:
            state = {'_py_factory': get_dotted_name(factory),
                     '_py_factory_args': self.get_state(args, seen)}
        for name, value in obj_state.items():
            state[name] = self.get_state(value, seen)
        return state

    def get_persistent_state(self, obj, seen):
        __traceback_info__ = obj
        # Persistent sub-objects are stored by reference, the key being
        # (collection name, oid).
        # Getting the collection name is easy, but if we have an unsaved
        # persistent object, we do not yet have an OID. This must be solved by
        # storing the persistent object.
        if obj._p_oid is None:
            dbref = self.store(obj, ref_only=True)
        else:
            db_name, coll_name = self.get_collection_name(obj)
            dbref = obj._p_oid
        # Create the reference sub-document. The _p_type value helps with the
        # deserialization later.
        return dbref

    def get_state(self, obj, seen=None):
        seen = seen or []
        if isinstance(obj, interfaces.MONGO_NATIVE_TYPES):
            # If we have a native type, we'll just use it as the state.
            return obj
        if isinstance(obj, str):
            # In Python 2, strings can be ASCII, encoded unicode or binary
            # data. Unfortunately, BSON cannot handle that. So, if we have a
            # string that cannot be UTF-8 decoded (luckily ASCII is a valid
            # subset of UTF-8), then we use the BSON binary type.
            try:
                obj.decode('utf-8')
                return obj
            except UnicodeError:
                return pymongo.binary.Binary(obj)

        # Some objects might not naturally serialize well and create a very
        # ugly Mongo entry. Thus, we allow custom serializers to be
        # registered, which can encode/decode different types of objects.
        for serializer in SERIALIZERS:
            if serializer.can_write(obj):
                return serializer.write(obj)

        if isinstance(obj, (type, types.ClassType)):
            # We frequently store class and function paths as meta-data, so we
            # need to be able to properly encode those.
            return {'_py_type': 'type',
                    'path': get_dotted_name(obj)}
        if isinstance(obj, (tuple, list, PersistentList)):
            # Make sure that all values within a list are serialized
            # correctly. Also convert any sequence-type to a simple list.
            return [self.get_state(value, seen) for value in obj]
        if isinstance(obj, (dict, PersistentDict)):
            # Same as for sequences, make sure that the contained values are
            # properly serialized.
            # Note: A big constraint in Mongo is that keys must be strings!
            has_non_string_key = False
            data = []
            for key, value in obj.items():
                data.append((key, self.get_state(value, seen)))
                has_non_string_key |= not isinstance(key, basestring)
            if not has_non_string_key:
                # The easy case: all keys are strings:
                return dict(data)
            else:
                # We first need to reduce the keys and then produce a data
                # structure.
                data = [(self.get_state(key), value) for key, value in data]
                return {'dict_data': data}

        if isinstance(obj, persistent.Persistent):
            # Only create a persistent reference, if the object does not want
            # to be a sub-document.
            if not getattr(obj, '_p_mongo_sub_object', False):
                return self.get_persistent_state(obj, seen)
            # This persistent object is a sub-document, so it is treated like
            # a non-persistent object.

        return self.get_non_persistent_state(obj, seen)

    def store(self, obj, ref_only=False):
        db_name, coll_name = self.get_collection_name(obj)
        coll = self._jar._conn[db_name][coll_name]
        if ref_only:
            # We only want to get OID quickly. Trying to reduce the full state
            # might cause infinite recusrion loop. (Example: 2 new objects
            # reference each other.)
            doc = {}
            # Make sure that the object gets saved fully later.
            self._jar.register(obj)
        else:
            # XXX: Handle newargs; see ZODB.serialize.ObjectWriter.serialize
            # Go through each attribute and search for persistent references.
            doc = self.get_state(obj.__getstate__())
        if getattr(obj, '_p_mongo_store_type', False):
            doc['_py_persistent_type'] = get_dotted_name(obj.__class__)
        # If conflict detection is turned on, store a serial number for the
        # document.
        if self._jar.detect_conflicts:
            doc['_py_serial'] = u64(getattr(obj, '_p_serial', 0)) + 1
            obj._p_serial = p64(doc['_py_serial'])

        if obj._p_oid is None:
            doc_id = coll.insert(doc)
            obj._p_jar = self._jar
            obj._p_oid = pymongo.dbref.DBRef(coll_name, doc_id, db_name)
            # Make sure that any other code accessing this object in this
            # session, gets the same instance.
            self._jar._object_cache[doc_id] = obj
        else:
            doc['_id'] = obj._p_oid.id
            coll.save(doc)
        return obj._p_oid


class ObjectReader(object):
    zope.interface.implements(interfaces.IObjectReader)

    def __init__(self, jar):
        self._jar = jar

    def simple_resolve(self, path):
        return resolve(path)

    def resolve(self, dbref):
        try:
            return OID_CLASS_LRU[dbref.id]
        except KeyError:
            pass
        # First we try to resolve the path directly.
        try:
            return self.simple_resolve(dbref.collection)
        except ImportError:
            pass
        # Let's now try to look up the path from the collection to path
        # mapping
        db = self._jar._conn[self._jar.default_database]
        coll = db[self._jar.name_map_collection]
        result = coll.find(
            {'collection': dbref.collection, 'database': dbref.database})
        if result.count() == 0:
            raise ImportError(dbref)
        elif result.count() == 1:
            # Do not add these results to the LRU cache, since the count might
            # change later.
            return self.simple_resolve(result.next()['path'])
        else:
            if dbref.id is None:
                raise ImportError(dbref)
            # Multiple object types are stored in the collection. We have to
            # look at the object to find out the type.
            obj_doc = self._jar._conn[dbref.database][dbref.collection].find_one(
                dbref.id, fields=('_py_persistent_type',))
            if '_py_persistent_type' in obj_doc:
                klass = self.simple_resolve(obj_doc['_py_persistent_type'])
            else:
                # Find the name-map entry where "doc_has_type" is False.
                for name_map_item in result:
                    if not name_map_item['doc_has_type']:
                        klass = self.simple_resolve(name_map_item['path'])
                        break
                else:
                    raise ImportError(path)
            OID_CLASS_LRU[dbref.id] = klass
            return klass

    def get_non_persistent_object(self, state, obj):
        if '_py_type' in state:
            # Handle the simplified case.
            klass = self.simple_resolve(state.pop('_py_type'))
            sub_obj = copy_reg._reconstructor(klass, object, None)
        elif '_py_persistent_type' in state:
            # Another simple case for persistent objects that do not want
            # their own document.
            klass = self.simple_resolve(state.pop('_py_persistent_type'))
            sub_obj = copy_reg.__newobj__(klass)
        else:
            factory = self.simple_resolve(state.pop('_py_factory'))
            factory_args = self.get_object(state.pop('_py_factory_args'), obj)
            sub_obj = factory(*factory_args)
        if len(state):
            sub_obj_state = self.get_object(state, obj)
            if isinstance(sub_obj, persistent.Persistent):
                sub_obj.__setstate__(sub_obj_state)
            else:
                sub_obj.__dict__.update(sub_obj_state)
        if getattr(sub_obj, '_p_mongo_sub_object', False):
            sub_obj._p_mongo_doc_object = obj
            sub_obj._p_jar = self._jar
        return sub_obj

    def get_object(self, state, obj):
        if isinstance(state, pymongo.objectid.ObjectId):
            # The object id is special. Preserve it.
            return state
        if isinstance(state, pymongo.binary.Binary):
            # Binary data in Python 2 is presented as a string. We will
            # convert back to binary when serializing again.
            return str(state)
        if isinstance(state, pymongo.dbref.DBRef):
            # Load a persistent object. Using the get_ghost() method, so that
            # caching is properly applied.
            return self.get_ghost(state)
        if isinstance(state, dict) and state.get('_py_type') == 'type':
            # Convert a simple object reference, mostly classes.
            return self.simple_resolve(state['path'])

        # Give the custom serializers a chance to weigh in.
        for serializer in SERIALIZERS:
            if serializer.can_read(state):
                return serializer.read(state)

        if isinstance(state, dict) and ('_py_factory' in state or \
               '_py_type' in state or '_py_persistent_type' in state):
            # Load a non-persistent object.
            return self.get_non_persistent_object(state, obj)
        if isinstance(state, (tuple, list)):
            # All lists are converted to persistent lists, so that their state
            # changes are noticed. Also make sure that all value states are
            # converted to objects.
            sub_obj = PersistentList(
                [self.get_object(value, obj) for value in state])
            sub_obj._p_mongo_doc_object = obj
            sub_obj._p_jar = self._jar
            return sub_obj
        if isinstance(state, dict):
            # All dictionaries are converted to persistent dictionaries, so
            # that state changes are detected. Also convert all value states
            # to objects.
            # Handle non-string key dicts.
            if 'dict_data' in state:
                items = state['dict_data']
            else:
                items = state.items()
            sub_obj = PersistentDict(
                [(self.get_object(name, obj), self.get_object(value, obj))
                 for name, value in items])
            sub_obj._p_mongo_doc_object = obj
            sub_obj._p_jar = self._jar
            return sub_obj
        return state

    def set_ghost_state(self, obj):
        # Look up the object state by coll_name and oid.
        coll = self._jar._conn[obj._p_oid.database][obj._p_oid.collection]
        doc = coll.find_one({'_id': obj._p_oid.id})
        doc.pop('_id')
        doc.pop('_py_persistent_type', None)
        # Store the serial, if conflict detection is enabled.
        if self._jar.detect_conflicts:
            obj._p_serial = p64(doc.pop('_py_serial', 0))
        # Now convert the document to a proper Python state dict.
        state = self.get_object(doc, obj)
        # Set the state.
        obj.__setstate__(dict(state))

    def get_ghost(self, dbref):
        # If we can, we return the object from cache.
        try:
            return self._jar._object_cache[dbref.id]
        except KeyError:
            pass
        klass = self.resolve(dbref)
        obj = klass.__new__(klass)
        obj._p_jar = self._jar
        obj._p_oid = dbref
        del obj._p_changed
        # Assign the collection after deleting _p_changed, since the attribute
        # is otherwise deleted.
        obj._p_mongo_database = dbref.database
        obj._p_mongo_collection = dbref.collection
        # Adding the object to the cache is very important, so that we get the
        # same object reference throughout the transaction.
        self._jar._object_cache[dbref.id] = obj
        return obj



##############################################################################
#
# Copyright (c) 2011 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Mongo Persistence Zope Containers"""
import UserDict
import persistent
import pymongo.dbref
import zope.component
from rwproperty import getproperty, setproperty
from zope.container import contained, sample
from zope.container.interfaces import IContainer

from mongopersist import interfaces, serialize
from mongopersist.zope import interfaces as zinterfaces
from mongopersist.datamanager import processSpec

class MongoContained(contained.Contained):

    @getproperty
    def __name__(self):
        return getattr(self, '_v_key', None)
    @setproperty
    def __name__(self, value):
        setattr(self, '_v_key', value)

    @getproperty
    def __parent__(self):
        return getattr(self, '_v_parent', None)
    @setproperty
    def __parent__(self, value):
        setattr(self, '_v_parent', value)


class SimpleMongoContainer(sample.SampleContainer, persistent.Persistent):

    def __getstate__(self):
        state = super(SimpleMongoContainer, self).__getstate__()
        state['data'] = state.pop('_SampleContainer__data')
        return state

    def __setstate__(self, state):
        # Mongopersist always reads a dictionary as persistent dictionary. And
        # modifying this dictionary will cause the persistence mechanism to
        # kick in. So we create a new object that we can easily modify without
        # harm.
        state = dict(state)
        state['_SampleContainer__data'] = state.pop('data', {})
        super(SimpleMongoContainer, self).__setstate__(state)

    def __getitem__(self, key):
        obj = super(SimpleMongoContainer, self).__getitem__(key)
        obj._v_key = key
        obj._v_parent = self
        return obj

    def get(self, key, default=None):
        '''See interface `IReadContainer`'''
        obj = super(SimpleMongoContainer, self).get(key, default)
        if obj is not default:
            obj._v_key = key
            obj._v_parent = self
        return obj

    def items(self):
        items = super(SimpleMongoContainer, self).items()
        for key, obj in items:
            obj._v_key = key
            obj._v_parent = self
        return items

    def values(self):
        return [v for k, v in self.items()]

    def __setitem__(self, key, object):
        super(SimpleMongoContainer, self).__setitem__(key, object)
        self._p_changed = True

    def __delitem__(self, key):
        super(SimpleMongoContainer, self).__delitem__(key)
        self._p_changed = True


class MongoContainer(contained.Contained,
                     persistent.Persistent,
                     UserDict.DictMixin):
    zope.interface.implements(IContainer, zinterfaces.IMongoContainer)
    _m_database = None
    _m_collection = None
    _m_mapping_key = 'key'
    _m_parent_key = 'parent'

    def __init__(self, collection=None, database=None,
                 mapping_key=None, parent_key=None):
        if collection:
            self._m_collection = collection
        if database:
            self._m_database = database
        if mapping_key is not None:
            self._m_mapping_key = mapping_key
        if parent_key is not None:
            self._m_parent_key = parent_key

    @property
    def _added(self):
        ann = self._m_jar.annotations.setdefault(self._p_oid or id(self), {})
        return ann.setdefault('added', {})

    @property
    def _deleted(self):
        ann = self._m_jar.annotations.setdefault(self._p_oid or id(self), {})
        return ann.setdefault('deleted', {})

    @property
    def _m_jar(self):
        # If the container is in a Mongo storage hierarchy, then getting the
        # datamanager is easy, otherwise we do an adapter lookup.
        if interfaces.IMongoDataManager.providedBy(self._p_jar):
            return self._p_jar
        else:
            provider = zope.component.getUtility(
                interfaces.IMongoDataManagerProvider)
            return provider.get()

    def get_collection(self):
        db_name = self._m_database or self._m_jar.default_database
        return self._m_jar._conn[db_name][self._m_collection]

    def _m_get_parent_key_value(self):
        if getattr(self, '_p_jar', None) is None:
            raise ValueError('_p_jar not found.')
        if interfaces.IMongoDataManager.providedBy(self._p_jar):
            return self
        else:
            return 'zodb-'+''.join("%02x" % ord(x) for x in self._p_oid).strip()

    def _m_get_items_filter(self):
        filter = {}
        # Make sure that we only look through objects that have the mapping
        # key. Objects not having the mapping key cannot be part of the
        # collection.
        if self._m_mapping_key is not None:
            filter[self._m_mapping_key] = {'$exists': True}
        if self._m_parent_key is not None:
            gs = self._m_jar._writer.get_state
            filter[self._m_parent_key] = gs(self._m_get_parent_key_value())
        return filter

    def __getitem__(self, key):
        if key in self._added:
            return self._added[key]
        if key in self._deleted:
            raise KeyError(key)
        filter = self._m_get_items_filter()
        filter[self._m_mapping_key] = key
        coll = self.get_collection()
        doc = coll.find_one(processSpec(coll, filter), fields=())
        if doc is None:
            raise KeyError(key)
        dbref = pymongo.dbref.DBRef(
            self._m_collection, doc['_id'],
            self._m_database or self._m_jar.default_database)
        obj = self._m_jar._reader.get_ghost(dbref)
        obj._v_key = key
        obj._v_parent = self
        return obj


    def _real_setitem(self, key, value):
        # This call by iteself caues the state to change _p_changed to True.
        setattr(value, self._m_mapping_key, key)
        if self._m_parent_key is not None:
            setattr(value, self._m_parent_key, self._m_get_parent_key_value())
        self._m_jar.register(value)
        # Temporarily store the added object, so it is immediately available
        # via the API.
        self._added[key] = value
        self._deleted.pop(key, None)

    def __setitem__(self, key, value):
        contained.setitem(self, self._real_setitem, key, value)

    def __delitem__(self, key):
        # Deleting the object from the database is not our job. We simply
        # remove it from the dictionary.
        value = self[key]
        if self._m_mapping_key is not None:
            delattr(value, self._m_mapping_key)
        if self._m_parent_key is not None:
            delattr(value, self._m_parent_key)
        self._deleted[key] = value
        self._added.pop(key, None)
        contained.uncontained(value, self, key)

    def keys(self):
        filter = self._m_get_items_filter()
        filter[self._m_mapping_key] = {'$ne': None}
        coll = self.get_collection()
        keys = [
            doc[self._m_mapping_key]
            for doc in coll.find(processSpec(coll, filter))
            if not doc[self._m_mapping_key] in self._deleted]
        keys += self._added.keys()
        return keys

    def raw_find(self, spec=None, *args, **kwargs):
        if spec is None:
            spec  = {}
        spec.update(self._m_get_items_filter())
        coll = self.get_collection()
        return coll.find(processSpec(coll, spec), *args, **kwargs)

    def find(self, spec=None, fields=None, *args, **kwargs):
        # If fields were not specified, we only request the oid and the key.
        fields = tuple(fields or ())
        fields += (self._m_mapping_key,)
        result = self.raw_find(spec, fields, *args, **kwargs)
        for doc in result:
            dbref = pymongo.dbref.DBRef(
                self._m_collection, doc['_id'],
                self._m_database or self._m_jar.default_database)
            obj = self._m_jar._reader.get_ghost(dbref)
            obj._v_key = doc[self._m_mapping_key]
            obj._v_parent = self
            yield obj

    def raw_find_one(self, spec_or_id=None, *args, **kwargs):
        if spec_or_id is None:
            spec_or_id  = {}
        if not isinstance(spec_or_id, dict):
            spec_or_id = {'_id': spec_or_id}
        spec_or_id.update(self._m_get_items_filter())
        coll = self.get_collection()
        return coll.find_one(processSpec(coll, spec_or_id), *args, **kwargs)

    def find_one(self, spec_or_id=None, fields=None, *args, **kwargs):
        # If fields were not specified, we only request the oid and the key.
        fields = tuple(fields or ())
        fields += (self._m_mapping_key,)
        doc = self.raw_find_one(spec_or_id, fields, *args, **kwargs)
        if doc is None:
            return None
        dbref = pymongo.dbref.DBRef(
            self._m_collection, doc['_id'],
            self._m_database or self._m_jar.default_database)
        obj = self._m_jar._reader.get_ghost(dbref)
        obj._v_key = doc[self._m_mapping_key]
        obj._v_parent = self
        return obj

class AllItemsMongoContainer(MongoContainer):
    _m_parent_key = None


class SubDocumentMongoContainer(MongoContained, MongoContainer):
    _p_mongo_sub_object = True



"""Setup
"""
import os
from setuptools import setup, find_packages

def read(*rnames):
    text = open(os.path.join(os.path.dirname(__file__), *rnames)).read()
    return unicode(text, 'utf-8').encode('ascii', 'xmlcharrefreplace')

setup (
    name='mongopersist',
    version='0.5.5',
    author = "Stephan Richter",
    author_email = "stephan.richter@gmail.com",
    description = "Mongo Persistence Backend",
    long_description=read('src', 'mongopersist', 'README.txt'),
    license = "ZPL 2.1",
    keywords = "mongo persistent ",
    classifiers = [
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Framework :: ZODB',
        'License :: OSI Approved :: Zope Public License',
        'Natural Language :: English',
        'Operating System :: OS Independent'],
    packages = find_packages('src'),
    package_dir = {'':'src'},
    extras_require = dict(
        test = (
            'zope.app.testing',
            'zope.testing',
            'ZODB3',
            ),
        zope = (
            'rwproperty',
            'zope.container',
            ),
        ),
    install_requires = [
        'transaction >=1.1.0',
        'lru',
        'pymongo',
        'setuptools',
        'zope.dottedname',
        'zope.interface',
    ],
    include_package_data = True,
    zip_safe = False,
    )



##############################################################################
#
# Copyright (c) 2011 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Thread-aware Mongo Connection Pool"""
from __future__ import absolute_import
import logging
import threading
import pymongo
import zope.interface

from mongopersist import datamanager, interfaces

log = logging.getLogger('mongopersist')

class MongoConnectionPool(object):
    """MongoDB connection pool contains the connection to a mongodb server.

    MongoConnectionPool is a global named utility, knows how to setup a
    thread (safe) shared mongodb connection instance.

    Note: pymongo offers connection pooling which we do not need since we use
    one connection per thread
    """
    zope.interface.implements(interfaces.IMongoConnectionPool)

    _mongoConnectionFactory = pymongo.Connection

    def __init__(self, host='localhost', port=27017, logLevel=20,
        tz_aware=True, connectionFactory=None):
        self.host = host
        self.port = port
        self.key = 'mongopersist-%s-%s' %(self.host, self.port)
        self.tz_aware = tz_aware
        if connectionFactory is not None:
            self._mongoConnectionFactory = connectionFactory
        self.logLevel = logLevel

    @property
    def storage(self):
        return LOCAL.__dict__

    def disconnect(self):
        conn = self.storage.get(self.key, None)
        if conn is not None:
            conn.disconnect()
        self.storage[self.key] = None

    @property
    def connection(self):
        conn = self.storage.get(self.key, None)
        if conn is None:
            self.storage[self.key] = conn = self._mongoConnectionFactory(
                self.host, self.port, tz_aware=self.tz_aware)
            if self.logLevel:
                log.log(self.logLevel, "Create connection for %s:%s" % (
                    self.host, self.port))

        return conn


LOCAL = threading.local()

class MongoDataManagerProvider(object):
    zope.interface.implements(interfaces.IMongoDataManagerProvider)

    def __init__(self, host='localhost', port=27017,
                 logLevel=20, tz_aware=True, **dm_kwargs):
        self.pool = MongoConnectionPool(host, port, logLevel, tz_aware)
        self.dm_kwargs = dm_kwargs

    def get(self):
        try:
            dm = LOCAL.data_manager
        except AttributeError, err:
            conn = self.pool.connection
            dm = LOCAL.data_manager = datamanager.MongoDataManager(
                conn, **self.dm_kwargs)
        return dm
